from pathlib import Path
import pyperclip

class ReturnHandler():   
    """
    ReturnHandler contains methods to return the Directory tree :
    
    @functions
    copyToClipboard     => Copies tree structure to clipboard.
    saveAsFile          => Saves tree structure to a file.
    
    """

    headDoc ="""
# Product Name

> Short blurb about what your product does.

## File Structure

```markdown
"""
    tailDoc ="""

```

## Contributing

1. Fork it (<https://github.com/yourname/yourproject/fork>)
2. Create your feature branch (`git checkout -b feature/fooBar`)
3. Commit your changes (`git commit -am 'Add some fooBar'`)
4. Push to the branch (`git push origin feature/fooBar`)
5. Create a new Pull Request
"""

    @staticmethod
    def copyToClipboard(tree_contents: list):
        """
        Saves Tree contents to clipboard.
        @param tree_contents: List object containing tree contents.
        """
        text = '\n'.join(map(str, tree_contents))
        pyperclip.copy(text)

    @staticmethod
    def saveAsFile(tree_contents: list, file_path: Path, readme: bool=False):
        """
        Saves Tree contents to file or readme.
        @param tree_contents: List object containing tree contents.
        @param file_path: Path to the file
        @param readme: True if file is readme
        """
        text = '\n'.join(map(str, tree_contents))
        if readme:
            with open(file_path, 'w', encoding="utf-8") as f:
                f.write(ReturnHandler.headDoc)
                f.write(text)
                f.write(ReturnHandler.tailDoc)
        else:
            with open(file_path, 'w', encoding="utf-8") as f:
                f.write(text)



from __init__ import __version__ as version
from docopt import docopt
from pathlib import Path
import sys

sys.dont_write_bytecode = True

class GetArgumentParser:
   
    """
    Argument Parser class for repostruc:
    
    @functions
    __init__     => Initializes docopt
    getArguments => Returns docopt
    
    """

    def __init__(self):
        """
        Initializes Usage of arguments for Docopt

        """
        doc = """repostruc.

                Usage:
                    repostruc 
                    repostruc (-c | --clip)
                    repostruc [-r | --readme] (-f | --file) [<filepath>]
                    repostruc (-h | --help)
                    repostruc (-v | --version)
                
                Options:
                    -r --readme   Add to readme file.
                    -c --clip     Copy to clipboard.
                    --version     Show version.

                """
        self.doc = doc
        self.version = version
        self.name = "repostruc"

    def getArguments(self):
        """
        Initializes Docopt.

        @return
        Docopt object for Argument Parsing 
        """
        return docopt(self.doc, version=f'{self.name}_{self.version}')


if __name__ == '__main__':
    argparse = GetArgumentParser()
    arguments = argparse.getArguments()
    print(arguments)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

# Package meta-data.
NAME = 'repostruc'
DESCRIPTION = 'Python module to generate Directory tree structure.'
URL = 'https://github.com/Atharva-Gundawar/repostruc'
EMAIL = 'atharva.n.gundawar@gmail.com'
AUTHOR = 'Atharva Gundawar'
REQUIRES_PYTHON = '>=3.5.0'
VERSION = '0.0.1'

REQUIRED = [
"pathlib","pyperclip","gitignore_parser","setuptools","io","docopt","pathlib"
]


here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=[
        'repostruc',
    ],
    package_dir={'repostruc':
                 'repostruc'},
    entry_points ={
            'console_scripts': [
                'repostruc = repostruc.repostruc:main'
            ]
    },
    include_package_data=True,
    install_requires=REQUIRED,
    keywords = 'Folder_structure folder structure repository directory tree python py',
    zip_safe = False,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        "Operating System :: OS Independent",
    ]
)


from gitignore_parser import parse_gitignore
from pathlib import Path
import pyperclip

class TreeHandler():
    """
    TreeHandler contains methods to handle the Directory tree :
    
    @functions
    getTree    => Returns visual tree structure.
    formatTree => Formats tree structure.
    
    """

    # prefix components:
    space =  '    '
    branch = '│   '
    # pointers:
    tee =    '├── '
    last =   '└── '

    ignore_files = ['.git','__pycache__','node_modules','.env']

    @staticmethod
    def getTree(dir_path: Path, prefix: str='', gitignore_path: Path='.gitignore'):
        """
        A recursive generator function,
        which yeilds a visual tree structure line by line.

        @param dir_path: Path object of targeted directory 
        @param prefix: Prefix for every level of the tree structure 
        @param ignore_files: Files and directories to ignore

        """    
        if Path(gitignore_path).is_file():
            matches_gitignore = parse_gitignore(gitignore_path)
            contents = [path for path in dir_path.iterdir() if not matches_gitignore(path.absolute()) and not path.name.endswith('.git')]
        else:
            contents = [path for path in dir_path.iterdir() if path.name not in TreeHandler.ignore_files]
        pointers = [TreeHandler.tee] * (len(contents) - 1) + [TreeHandler.last]
        for pointer, path in zip(pointers, contents):
            yield prefix + pointer + path.name
            if path.is_dir(): 
                extension = TreeHandler.branch if pointer == TreeHandler.tee else TreeHandler.space 
                yield from TreeHandler.getTree(path, prefix=prefix+extension,gitignore_path=gitignore_path)

    @staticmethod
    def formatTree(dir_path: Path, gitignore_path: Path='.gitignore', dsc_spacing: int=4, base_spacing: int=2):
        """
        Adds spacing before and after every file.

        @param dir_path: Path object of targeted directory.
        @param dsc_spacing: Spacing between file and discription.
        @param base_spacing: Spacing before file. 

        @return List of tree contents.
        """
        tree_contents = [line for line in TreeHandler.getTree(Path().absolute(),gitignore_path=gitignore_path)]
        max_string_length = max(tree_contents, key=len)
        tree_contents = [' '*base_spacing + line + ' '*int(dsc_spacing + len(max_string_length) - len(line)) + '<- DSC' for line in tree_contents]
        return tree_contents



from argHandler import GetArgumentParser
from returnUtils import ReturnHandler
from treeUtils import TreeHandler
from pathlib import Path

def main():
    """
    Main function which handles argumenst passed,
    and calls respective functions. 

    """
    argparse = GetArgumentParser()
    arguments = argparse.getArguments()
    cur_dir = Path.cwd()
    if arguments['--clip']:
        ReturnHandler.copyToClipboard(TreeHandler.formatTree(cur_dir))
    elif arguments['-f'] or arguments['--file']:
        file_path = Path.absolute(Path(arguments['<filepath>'])) if arguments['<filepath>'] else Path('PROJECTINFO.md') 
        ReturnHandler.saveAsFile(
            TreeHandler.formatTree(cur_dir),
            file_path,
            arguments['--readme']
            )
    else:
        print(argparse.doc)


if __name__ == '__main__' :
    main()